@@ -10,20 +10,24 @@ import plotly.graph_objs as go
 from dataclasses import dataclass
 from typeguard import typechecked
 
-from relevanceai.api.client import APIClient
+from relevanceai.api.client import BatchAPIClient
 from relevanceai.base import Base
-from relevanceai.visualise.constants import *
+from relevanceai.vector_tools.constants import DIM_REDUCTION, CLUSTER
 
-from relevanceai.visualise.cluster import cluster, ClusterBase
-from relevanceai.visualise.dim_reduction import dim_reduce, DimReductionBase
+from relevanceai.vector_tools.cluster import Cluster, ClusterBase
+from relevanceai.vector_tools.dim_reduction import DimReduction, DimReductionBase
+
+from typing import List, Union, Dict, Any, Tuple, Optional
+from typing_extensions import Literal
 
 from doc_utils import DocUtils
 
 RELEVANCEAI_BLUE = "#1854FF"
+MARKER_SIZE = 5
 
 
 @dataclass
-class Projector(APIClient, Base, DocUtils):
+class Projector(BatchAPIClient, Base, DocUtils):
     """
     Projector class.
 
@@ -42,42 +46,41 @@ class Projector(APIClient, Base, DocUtils):
                 )
     """
 
-    def __init__(self, project, api_key, base_url):
-        self.base_args = {
-            "project": project,
-            "api_key": api_key,
-            "base_url": base_url,
-        }
-        super().__init__(**self.base_args)
+    def __init__(self, project, api_key):
+        self.project = project
+        self.api_key = api_key
+        super().__init__(project, api_key)
 
     @typechecked
     def plot(
         self,
         dataset_id: str,
         vector_field: str,
-        number_of_points_to_render: Optional[int] = 1000,
+        number_of_points_to_render: int = 1000,
         random_state: int = 0,
-        ### Dimensionality reduction args
+        # Dimensionality reduction args
         dr: Union[DIM_REDUCTION, DimReductionBase] = "pca",
         dr_args: Union[None, Dict] = None,
         # TODO: Add support for 2
-        dims: Literal[3] = 3,
-        ### Plot rendering args
+        dims: Literal[2, 3] = 3,
+        # Plot rendering args
         vector_label: Union[None, str] = None,
         vector_label_char_length: Union[None, int] = 50,
         colour_label: Union[None, str] = None,
         colour_label_char_length: Union[None, int] = 20,
         hover_label: List[str] = [],
-        ### Cluster args
+        # Cluster args
         cluster: Union[CLUSTER, ClusterBase] = None,
         cluster_args: Union[None, Dict] = None,
         num_clusters: Union[None, int] = 10,
+        marker_colour: str = RELEVANCEAI_BLUE,
+        marker_size: int = MARKER_SIZE
     ):
         """
         Plot function for Embedding Projector class
 
         To write your own custom dimensionality reduction, you should inherit from DimReductionBase:
-        from relevanceai.visualise.dim_reduction import DimReductionBase 
+        from relevanceai.visualise.dim_reduction import DimReductionBase
         class CustomDimReduction(DimReductionBase):
             def fit_transform(self, vectors):
                 return np.arange(512, 2)
@@ -99,6 +102,7 @@ class Projector(APIClient, Base, DocUtils):
         self.dataset_id = dataset_id
         self.vector_label = vector_label
         self.vector_field = vector_field
+        self.number_of_points_to_render = number_of_points_to_render
         self.random_state = random_state
         self.vector_label_char_length = vector_label_char_length
         self.colour_label = colour_label
@@ -112,47 +116,55 @@ class Projector(APIClient, Base, DocUtils):
         self.cluster_args = cluster_args
 
         if (vector_label is None) and (colour_label is None):
-            warnings.warn(f"A vector_label or colour_label has not been specified.")
+            warnings.warn(
+                f"A vector_label or colour_label has not been specified.")
 
         if number_of_points_to_render and number_of_points_to_render > 1000:
-            warnings.warn( f"You are rendering over 1000 points, this may take some time ...")
+            warnings.warn(
+                f"You are rendering over 1000 points, this may take some time ..."
+            )
 
         number_of_documents = number_of_points_to_render
         self.vector_fields = self._get_vector_fields()
 
-        labels = ["_id", vector_field, vector_label, colour_label] 
-        if hover_label: labels += hover_label
-        fields = [ label for label in labels if label ] 
-        self.docs = self._retrieve_documents(
-            dataset_id, fields, number_of_documents, page_size=1000
+        labels = ["_id", vector_field, vector_label, colour_label]
+        if hover_label:
+            labels += hover_label
+        fields = [label for label in labels if label]
+        self.docs = self.get_documents(
+            dataset_id, number_of_documents=number_of_documents, batch_size=1000, select_fields=fields
         )
         self._remove_empty_vector_fields(vector_field)
-        
-        return self.plot_from_docs(self.docs)
-   
 
+        return self.plot_from_docs(self.docs, self.dims, marker_size, marker_colour)
 
-    def plot_from_docs(self, docs: List[Dict[str, Any]], *args, **kwargs):
+    def plot_from_docs(self, docs: List[Dict[str, Any]], dims: int, marker_size: int, marker_colour: str, *args, **kwargs):
         for k, v in kwargs.items():
             setattr(self, k, v)
-        self.vectors = np.array(self.get_field_across_documents(self.vector_field, docs))
-        self.vectors_dr = dim_reduce(vectors=self.vectors, dr=self.dr, dr_args=self.dr_args, dims=self.dims)
 
         if self._is_valid_vector_name(self.vector_field):
-            
-            self.vectors_dr = dim_reduce(vectors=self.vectors, dr=self.dr, dr_args=self.dr_args, dims=self.dims)
+
+            self.vectors = np.array(
+                self.get_field_across_documents(self.vector_field, docs)
+            )
+            self.vectors_dr = DimReduction.dim_reduce(
+                vectors=self.vectors, dr=self.dr, dr_args=self.dr_args, dims=self.dims
+            )
             points = {
                 "x": self.vectors_dr[:, 0],
                 "y": self.vectors_dr[:, 1],
-                "z": self.vectors_dr[:, 2],
-                "_id": self.get_field_across_documents("_id", docs)
+                "_id": self.get_field_across_documents("_id", docs),
             }
-            
+            if dims == 3:
+                points["z"] = self.vectors_dr[:, 2]
+
             self.embedding_df = pd.DataFrame(points)
             if self.hover_label and all(
                 self._is_valid_label_name(l) for l in self.hover_label
             ):
-                self.embedding_df = pd.concat([self.embedding_df, pd.DataFrame(docs)], axis=1)
+                self.embedding_df = pd.concat(
+                    [self.embedding_df, pd.DataFrame(docs)], axis=1
+                )
 
             if self.vector_label and self._is_valid_label_name(self.vector_label):
                 self.labels = self.get_field_across_documents(
@@ -180,13 +192,17 @@ class Projector(APIClient, Base, DocUtils):
                 #     k=self.num_clusters,
                 # )
                 # self.cluster_labels = _cluster.cluster_labels
-                self.cluster_labels = cluster(vectors=self.vectors, cluster=self.cluster, cluster_args=self.cluster_args)
+                self.cluster_labels = Cluster.cluster(
+                    vectors=self.vectors,
+                    cluster=self.cluster,
+                    cluster_args=self.cluster_args,
+                )
                 self.embedding_df["cluster_labels"] = self.cluster_labels
                 self.legend = "cluster_labels"
 
             self.embedding_df.index = self.embedding_df["_id"]
             return self._generate_fig(
-                embedding_df=self.embedding_df, legend=self.legend
+                embedding_df=self.embedding_df, legend=self.legend, marker_size=marker_size, marker_colour=marker_colour
             )
 
     def _get_vector_fields(self) -> List[str]:
@@ -207,7 +223,8 @@ class Projector(APIClient, Base, DocUtils):
             else:
                 raise ValueError(f"{vector_name} is not a valid vector name")
         else:
-            raise ValueError(f"{vector_name} is not in the {self.dataset_id} schema")
+            raise ValueError(
+                f"{vector_name} is not in the {self.dataset_id} schema")
 
     def _is_valid_label_name(self, label_name: str) -> bool:
         """
@@ -221,7 +238,8 @@ class Projector(APIClient, Base, DocUtils):
             else:
                 raise ValueError(f"{label_name} is not a valid label name")
         else:
-            raise ValueError(f"{label_name} is not in the {self.dataset_id} schema")
+            raise ValueError(
+                f"{label_name} is not in the {self.dataset_id} schema")
 
     def _remove_empty_vector_fields(self, vector_field: str) -> List[Dict]:
         """
@@ -230,80 +248,21 @@ class Projector(APIClient, Base, DocUtils):
         self.docs = [d for d in self.docs if d.get(vector_field)]
         return self.docs
 
-    def _retrieve_documents(
-        self,
-        dataset_id: str,
-        fields: List[str],
-        number_of_documents: Optional[int] = 1000,
-        page_size: int = 1000,
-        filters=[],
-    ) -> List[Dict]:
-        """
-        Retrieve all documents from dataset
-        """
-        if number_of_documents:
-            if page_size > number_of_documents or self.random_state != 0:
-                page_size = number_of_documents  # type: ignore
-        else:
-            number_of_documents = 999999999999999
-
-        is_random = True if self.random_state != 0 else False
-        resp = self.datasets.documents.get_where(
-            dataset_id=dataset_id,
-            select_fields=fields,
-            include_vector=True,
-            page_size=page_size,
-            is_random=is_random,
-            random_state=self.random_state,
-            filters=filters,
-        )
-        data = resp["documents"]
-
-        if (
-            (number_of_documents > page_size)
-            and (is_random == False)
-            and (self.random_state == 0)
-        ):
-            _cursor = resp["cursor"]
-            _page = 0
-            while resp:
-                self.logger.debug(f"Paginating {_page} page size {page_size} ...")
-                resp = self.datasets.documents.get_where(
-                    dataset_id=dataset_id,
-                    select_fields=fields,
-                    page_size=page_size,
-                    cursor=_cursor,
-                    include_vector=True,
-                    verbose=True,
-                    filters=filters,
-                )
-                _data = resp["documents"]
-                _cursor = resp["cursor"]
-                if (_data == []) or (_cursor == []):
-                    break
-                data += _data
-                if number_of_documents and (len(data) >= int(number_of_documents)):
-                    break
-                _page += 1
-            data = data[:number_of_documents]
-
-        self.docs = data
-        return self.docs
-
     def _generate_fig(
         self,
         embedding_df: pd.DataFrame,
+        marker_size: int,
+        marker_colour: str,
         legend: Union[None, str],
     ) -> go.Figure:
-
         """
-        Generates the 3D scatter plot
+        Generates the Scatter plot
         """
-        plot_title = f"<b>3D Embedding Projector Plot<br>Dataset Id: {self.dataset_id} - {len(embedding_df)} points<br>Vector Field: {self.vector_field}<br></b>"
-        self.hover_label = ['_id'] + self.hover_label
+        plot_title = f"<b>{self.dims}D Embedding Projector Plot<br>Dataset Id: {self.dataset_id} - {len(embedding_df)} points<br>Vector Field: {self.vector_field}<br></b>"
+        self.hover_label = ["_id"] + self.hover_label
         text_labels = None
         plot_mode = "markers"
-        
+
         """
         Generates data for word plot
         If vector_label set, generates text_labels, otherwise shows points only
@@ -322,13 +281,13 @@ class Projector(APIClient, Base, DocUtils):
                 text_labels = embedding_df["labels"].apply(
                     lambda x: x[: self.vector_label_char_length] + "..."
                 )
-            
+
             self.hover_label.insert(1, self.vector_label)
 
             # self.hover_label = [self.vector_label] + self.hover_label
             # self.hover_label = list(set(self.hover_label))
 
-            ## TODO: We can change this later to show top 100 neighbours of a selected word
+            # TODO: We can change this later to show top 100 neighbours of a selected word
             #  # Regular displays the full scatter plot with only circles
             # if wordemb_display_mode == 'regular':
             #     plot_mode = 'markers'
@@ -350,26 +309,39 @@ class Projector(APIClient, Base, DocUtils):
             #     neighbors_idx = nearest_neighbours[:100].index
             #     embedding_df =  embedding_df.loc[neighbors_idx]
 
-    
-        custom_data, hovertemplate = self._generate_hover_template(df=embedding_df)
-        scatter = go.Scatter3d(
-            x=embedding_df["x"],
-            y=embedding_df["y"],
-            z=embedding_df["z"],
-            text=text_labels,
-            textposition="middle center",
-            showlegend=False,
-            mode=plot_mode,
-            marker = {
-                "size": 3,
-                "color": RELEVANCEAI_BLUE,
-                "symbol": "circle"
-            },
-            customdata=custom_data,
-            hovertemplate=hovertemplate,
+        custom_data, hovertemplate = self._generate_hover_template(
+            df=embedding_df, dims=self.dims
         )
-        data = [ scatter ]
-        
+
+        vector_label_scatter_args = {
+            "text": text_labels,
+            "textposition": "middle center",
+            "showlegend": False,
+            "mode": plot_mode,
+            "marker": {
+                "size": marker_size,
+                "color": marker_colour,
+                "symbol": "circle",
+            },
+            "customdata": custom_data,
+            "hovertemplate": hovertemplate,
+        }
+
+        if self.dims == 3:
+            scatter = go.Scatter3d(
+                x=embedding_df["x"],
+                y=embedding_df["y"],
+                z=embedding_df["z"],
+                **vector_label_scatter_args,
+            )
+
+        else:
+            scatter = go.Scatter(
+                x=embedding_df["x"], y=embedding_df["y"], **vector_label_scatter_args
+            )
+
+        data = [scatter]
+
         """
         Generates data for colour plot if selected
         """
@@ -389,7 +361,6 @@ class Projector(APIClient, Base, DocUtils):
 
             self.hover_label.insert(1, self.colour_label)
 
-            
             data = []
             groups = embedding_df.groupby(legend)
             for idx, val in groups:
@@ -407,25 +378,39 @@ class Projector(APIClient, Base, DocUtils):
 
                 #     self.hover_label = [self.vector_label] + self.hover_label
                 #     self.hover_label = list(set(self.hover_label))
-                    
-                custom_data, hovertemplate = self._generate_hover_template(df=val)
-                scatter = go.Scatter3d(
-                    name=idx,
-                    x=val["x"],
-                    y=val["y"],
-                    z=val["z"],
-                    # text=[ idx for _ in range(val["x"].shape[0]) ],
-                    textposition="top center",
-                    text=text_labels,
-                    mode=plot_mode,
-                    marker={
-                        "size": 3, "symbol": "circle"
-                    },
-                    customdata=custom_data,
-                    hovertemplate=hovertemplate,
-                )
-                data.append(scatter)
 
+                custom_data, hovertemplate = self._generate_hover_template(
+                    df=val, dims=self.dims
+                )
+
+                colour_label_scatter_args = (
+                    {  # text:[ idx for _ in range(val["x"].shape[0]) ],
+                        "text": text_labels,
+                        "textposition": "top center",
+                        "showlegend": False,
+                        "mode": plot_mode,
+                        "marker": {"size": marker_size, "symbol": "circle"},
+                        "customdata": custom_data,
+                        "hovertemplate": hovertemplate,
+                    }
+                )
+
+                if self.dims == 3:
+                    scatter = go.Scatter3d(
+                        name=idx,
+                        x=val["x"],
+                        y=val["y"],
+                        z=val["z"],
+                        **colour_label_scatter_args,
+                    )
+
+                else:
+
+                    scatter = go.Scatter(
+                        name=idx, x=val["x"], y=val["y"], **colour_label_scatter_args
+                    )
+
+                data.append(scatter)
 
         """
         Generating figure
@@ -434,20 +419,11 @@ class Projector(APIClient, Base, DocUtils):
             "title": "",
             "showgrid": True,
             "zeroline": False,
-            "showticklabels": False
+            "showticklabels": False,
         }
         layout = go.Layout(
-            margin={
-                "l": 0,
-                "r": 0,
-                "b": 0,
-                "t": 0
-            },
-            scene={
-                "xaxis": axes,
-                "yaxis": axes,
-                "zaxis": axes
-            }
+            margin={"l": 0, "r": 0, "b": 0, "t": 0},
+            scene={"xaxis": axes, "yaxis": axes, "zaxis": axes},
         )
 
         if self.cluster:
@@ -478,21 +454,32 @@ class Projector(APIClient, Base, DocUtils):
         return fig
 
     def _generate_hover_template(
-        self, df: pd.DataFrame
+        self, df: pd.DataFrame, dims: int
     ) -> Tuple[Optional[pd.DataFrame], Optional[str]]:
         """
         Generating hover template
         """
         self.hover_label = list(sorted(set(self.hover_label)))
-        custom_data = df[ self.hover_label ]
+        custom_data = df[self.hover_label]
         custom_data_hover = [
             f"{c}: %{{customdata[{i}]}}"
             for i, c in enumerate(self.hover_label)
             if self._is_valid_label_name(c)
         ]
-        hovertemplate = "<br>".join([
-                "X: %{x}   Y: %{y}   Z: %{z}",
-                ] + custom_data_hover 
-                ) + "<extra></extra>"
-                
+
+        if dims == 2:
+            coord_info = "X: %{x}   Y: %{y}"
+        else:
+            coord_info = "X: %{x}   Y: %{y}   Z: %{z}"
+
+        hovertemplate = (
+            "<br>".join(
+                [
+                    coord_info,
+                ]
+                + custom_data_hover
+            )
+            + "<extra></extra>"
+        )
+
         return custom_data, hovertemplate
